---
{% for k, v in front_matter.items() %}
{{ k |lower |snake_case }}: {{ v }}
{% endfor %}
---
<!--
THIS FILE IS GENERATED BY NOTOMA AUTOMATICALLY, DON'T EDIT IT!
Notion link for this article: {{ page | notion_url}}
-->

{% for block in page.children %}
{% set block_template = block | template_name %}
{% if block_template is not none %}
    {% include block_template %}
{% else %}
    {{ block | render_block }}
{% endif %}

{% endfor %}
