{%- extends 'markdown.tpl' -%}

{% block body %}
{% if resources['meta'] | length > 0 and resources['front_matter'] is true %}
---
{%- for k, v in resources['meta'].items() %}
{{ k }}: {{ v -}}
{% endfor %}
---
{% endif %}
<!--
THIS FILE IS AUTOGENERATED, DON'T EDIT.
File to edit instead: {{resources['nb_path']}}
-->
{{ super() }}
{%- endblock body %}

{% block codecell -%}
<div class="codecell" markdown="1">
{{ super() }}
</div>
{% endblock codecell %}

{% block input_group -%}
<div class="input_area" markdown="1">
{{ super() }}
</div>
{% endblock input_group %}

{% block output_group -%}
<div class="output_area" markdown="1">
{{ super() }}
</div>
{% endblock output_group %}

