# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['notoma']

package_data = \
{'': ['*'], 'notoma': ['templates/*', 'templates/blocks/*']}

install_requires = \
['click', 'jinja2', 'notion>=0.0.25', 'python-dotenv']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata>=1.0,<2.0']}

entry_points = \
{'console_scripts': ['notoma = notoma.cli:runner',
                     'notoma-dev = notoma.dev:cli']}

setup_kwargs = {
    'name': 'notoma',
    'version': '0.2.2',
    'description': 'Write and publish your blog articles from Notion.',
    'long_description': '<!--\nTHIS FILE IS AUTOGENERATED, DON\'T EDIT.\nFile to edit instead: notebooks/index.ipynb\n-->\n\n# Notoma\n\nWrite articles for your static gen blog in Notion.\n\n\n\n<a href="https://codeclimate.com/github/xnutsive/notoma/maintainability"><img src="https://api.codeclimate.com/v1/badges/70943357e5d2c54c153a/maintainability" /></a>\n<a href="https://pypi.org/project/notoma/"><img src="https://img.shields.io/pypi/v/notoma" alt="pypi" /></a>\n![Linters](https://github.com/xnutsive/notoma/workflows/Linters/badge.svg)\n\n- [Documentation website](https://xnutsive.github.io/notoma/)\n    - [Using the CLI](https://xnutsive.github.io/notoma/using-the-cli)\n    - [Contributing](https://xnutsive.github.io/notoma/contributing)\n    - [Supported Markdown Tags](https://xnutsive.github.io/notoma/supported-markdown-tags) \n\n---\n## Install\n\nNotoma is available via Pip or Homebrew: \n\n```bash\n# Installing with pip, use this if you plan using Notoma as a python library.\npip install notoma\n```\n\nInstalling with Homebrew on Mac OS.\n\n```bash\nbrew install xnutsive/notoma/notoma\n```\n\n---\n## What can you do with Notoma\nNotoma provides commands to: \n- Convert contents of your Notion Blog database to a bunch of Markdown files.\n- *Coming soon*: Watch Notion Blog database for updates and regenerate Markdown files on any updates.\n- *Coming soon*: Create a new Notion database for your Blog with all required fields.\n\nBasic usage example: this command will convert only published posts from a Notion blog database to the `./posts/ directory`.\n\n```bash\nnotoma convert --dest ./posts/\n```\n\nThis example assumes that you have a `.env` config file with authentication and blog url parameters in it.\n\n#### Authenticating in Notion\n\nNotoma uses an internal Notion API, and that, unfortunately, requires you to provide an authentication token `token_v2` that you can find in your notion.so cookes.\n\nYou can provide `token_v2` option to every command line call, or store it in your environment, or [`.env` config file](.env.sample).\n\n---\n## Notion database structure\nNotoma has very few expectations about how your Notion is structured. Here\'s a [public example database](https://www.notion.so/respawn/7b46cea379bd4d45b68860c2fa35a2d4?v=b4609f6aae0d4fc1adc65a73f72d0e21).\n\nNotoma requires that your Notion blog database has the following **properties**:\n- **Published**: whether the article is published, or is still a draft\n- **Title**: Will be used to create a file name for that article\'s Markdown equivalent file. *Won\'t be used in the article itself.*\n\nNotoma tries to parse other properties and add them as front matter into the resulting Markdown articles: \n- **Published at** will be used as publicataion date for the article, if present.\n- **Categories** will be used as `categories` front matter key, so it\'s expected to be a **multiple choice** propery.\n\n',
    'author': 'Nate Gadzhibalaev',
    'author_email': 'nate@respawn.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://xnutsive.github.io/notoma',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)
